@@ -1,8 +1,8 @@
 typedef struct
 {
-  const gchar const *locale;
-  const gchar const *type;
-  const gchar const *id;
+  const gchar *const locale;
+  const gchar *const type;
+  const gchar *const id;
 } DefaultInputSource;
 
 static DefaultInputSource default_input_sources[] =
@@ -33,8 +33,9 @@ static DefaultInputSource default_input_sources[] =
   { "gu_IN",    "ibus",         "m17n:gu:inscript" },
   { "he_IL",    "xkb",          "il" },
   { "hi_IN",    "ibus",         "m17n:hi:inscript" },
+  { "id_ID",    "xkb",          "us" },
   { "it_IT",    "xkb",          "it" },
-  { "ja_JP",    "ibus",         "kkc" },
+  { "ja_JP",    "ibus",         "anthy" },
   { "kn_IN",    "ibus",         "m17n:kn:kgp" },
   { "ko_KR",    "ibus",         "hangul" },
   { "mai_IN",   "ibus",         "m17n:mai:inscript" },
@@ -53,7 +54,7 @@ static DefaultInputSource default_input_sources[] =
   { "te_IN",    "ibus",         "m17n:te:inscript" },
   { "ur_IN",    "ibus",         "m17n:ur:phonetic" },
   { "zh_CN",    "ibus",         "libpinyin" },
-  { "zh_HK",    "ibus",         "cangjie" },
+  { "zh_HK",    "ibus",         "table:cangjie5" },
   { "zh_TW",    "ibus",         "libzhuyin" },
   { NULL,       NULL,           NULL }
 };
