@@ -33,9 +33,6 @@
 #include <glib.h>
 #include <glib/gstdio.h>
 
-#include <X11/Xlib.h>
-#include <gdk/gdkx.h>
-
 #include "gnome-rr-config.h"
 
 #include "gnome-rr-private.h"
@@ -661,7 +658,7 @@ gnome_rr_config_apply_persistent (GnomeRRConfig *config,
 /**
  * gnome_rr_config_get_outputs:
  *
- * Returns: (array zero-terminated=1) (element-type GnomeDesktop.RROutputInfo) (transfer none): the output configuration for this #GnomeRRConfig
+ * Returns: (array zero-terminated=1) (element-type GnomeRROutputInfo) (transfer none): the output configuration for this #GnomeRRConfig
  */
 GnomeRROutputInfo **
 gnome_rr_config_get_outputs (GnomeRRConfig *self)
